"""
Digital Storage Oscilloscope Common Functions
 _____  _____   ____ _______ ____ _________     _______  ______
|  __ ||  __ | / __ |__   __/ __ |__   __| |   / /  __ ||  ____|
| |__) | |__) | |  | | | | | |  | | | |   | |_/ /| |__) | |__
|  ___/|  _  /| |  | | | | | |  | | | |    |   / |  ___/|  __|
| |    | | | || |__| | | | | |__| | | |     | |  | |    | |____
|_|    |_|  |_||____/  |_|  |____/  |_|     |_|  |_|    |______|
                        _            _           _
                       | |          | |         | |
            _   _ _ __ | |_ ___  ___| |_ ___  __| |
           | | | | '_ || __/ _ |/ __| __/ _ |/ _` |
           | |_| | | | | ||  __/|__ | ||  __/ (_| |
            |__,_|_| |_||__|___||___/|__|___||__,_|
"""
from rssd.yaVISA import jaVisa

class DSO(jaVisa):
    """ Rohde & Schwarz Digital Storage Oscilloscope Object """
    def __init__(self):
        super(DSO, self).__init__()
        self.Model = "DSO"

#####################################################################
### DSO Get Functions
#####################################################################
    def Get_AcqTime(self):
        """ Seconds """
        rdStr = self.query(f':TIM:RANG?')
        return rdStr

    def Get_ChState(self, ch=1):
        rdStr = self.query(f':chAN{ch}:STAT?')
        return rdStr

    def Get_SamplingRate(self):
        """ MHz? """
        rdStr = self.query(f':ACQ:SRR?')
        return rdStr

    def Get_TimeRes(self):
        """ Seconds """
        rdStr = self.query(f':ACQ:RES?')
        return rdStr

    def Get_TimeScale(self):
        """ Seconds """
        rdStr = self.query(f':TIM:SCAL?')
        return rdStr

    def Get_Trace(self,ch,Wave):
        """ ASCII Trace """
        self.write(f'FORM ASCII')
        YVals = self.query(f':chAN{ch}:WAV{Wave}:DATA?')
        Headr = self.queryFloatArry(f':chAN{ch}:WAV{Wave}:DATA:HEAD?')
        Start = Headr[0]
        Stops = Headr[1]
        Sampl = Headr[2]
        return YVals

###############################################################################
###  DSO Get Cursor Functions
###############################################################################

    def Get_Cursor_X_Delta(self, ch=1):
        rdStr = self.query(f':CURS{ch}:XDEL:VAL?')
        return rdStr

    def Get_Cursor_X_Delta_Inverse(self, ch=1):
        rdStr = self.query(f':CURS{ch}:XDEL:INV?')
        return rdStr

    def Get_Cursor_Y_Delta(self, ch=1):
        rdStr = self.query(f':CURS{ch}:YDEL:VAL?')
        return rdStr

    def Get_Cursor_Y_Delta_Slope(self, ch=1):
        rdStr = self.query(f':CURS{ch}:YDEL:SLOP?')
        return rdStr

###############################################################################
###  DSO Get Measurement Functions
###############################################################################

    def Get_Meas_Res_Actual(self, mode, ch=1):
        """ HIGH LOW AMP MAX MIN PDELta MEAN RMS STDD POVershoot 
            AREA RTIMe FTIMe PPULse PER FREQ PULCnt DELay ..."""
        rdStr = self.query(f':MEAS{ch}:RES:ACT? {mode}')
        return rdStr

    def Get_Meas_Res_Average(self, mode, ch=1):
        """ HIGH LOW AMP MAX MIN PDELta MEAN RMS STDD POVershoot 
            AREA RTIMe FTIMe PPULse PER FREQ PULCnt DELay ..."""
        rdStr = self.query(f':MEAS{ch}:RES:AVG? {mode}')
        return rdStr

    def Get_Meas_Res_Event_Count(self, mode, ch=1):
        """ HIGH LOW AMP MAX MIN PDELta MEAN RMS STDD POVershoot 
            AREA RTIMe FTIMe PPULse PER FREQ PULCnt DELay ..."""
        rdStr = self.query(f':MEAS{ch}:RES:EVTC? {mode}')
        return rdStr

    def Get_Meas_Res_Negetive_Peak(self, mode, ch=1):
        """ HIGH LOW AMP MAX MIN PDELta MEAN RMS STDD POVershoot 
            AREA RTIMe FTIMe PPULse PER FREQ PULCnt DELay ..."""
        rdStr = self.query(f':MEAS{ch}:RES:NPE? {mode}')
        return rdStr

    def Get_Meas_Res_Positive_Peak(self, mode, ch=1):
        """ HIGH LOW AMP MAX MIN PDELta MEAN RMS STDD POVershoot 
            AREA RTIMe FTIMe PPULse PER FREQ PULCnt DELay ..."""
        rdStr = self.query(f':MEAS{ch}:RES:PPE? {mode}')
        return rdStr

    def Get_Meas_Res_Reliability(self, mode, ch=1):
        """ HIGH LOW AMP MAX MIN PDELta MEAN RMS STDD POVershoot 
            AREA RTIMe FTIMe PPULse PER FREQ PULCnt DELay ..."""
        rdStr = self.query(f':MEAS{ch}:RES:REL? {mode}')
        return rdStr

    def Get_Meas_Res_RMS(self, mode, ch=1):
        """ HIGH LOW AMP MAX MIN PDELta MEAN RMS STDD POVershoot 
            AREA RTIMe FTIMe PPULse PER FREQ PULCnt DELay ..."""
        rdStr = self.query(f':MEAS{ch}:RES:RMS? {mode}')
        return rdStr

    def Get_Meas_Res_Wave_Count(self, mode, ch=1):
        """ HIGH LOW AMP MAX MIN PDELta MEAN RMS STDD POVershoot 
            AREA RTIMe FTIMe PPULse PER FREQ PULCnt DELay ..."""
        rdStr = self.query(f':MEAS{ch}:RES:WFMC? {mode}')
        return rdStr

    def Get_Meas_Res_Std_Dev(self, mode, ch=1):
        """ HIGH LOW AMP MAX MIN PDELta MEAN RMS STDD POVershoot 
            AREA RTIMe FTIMe PPULse PER FREQ PULCnt DELay ..."""
        rdStr = self.query(f':MEAS{ch}:RES:STDD? {mode}')
        return rdStr  

#####################################################################
### DSO Init Functions
#####################################################################
    def Init_Measurement(self):
        #Configure instrument measurment
        pass

#####################################################################
### DSO Set Functions
#####################################################################
    def Set_AcqTime(self, sec):
        """ Seconds """
        self.write(f':TIM:RANG {sec}')

    def Set_Autoset(self):
        self.write(f':AUT')

    def Set_ChCoupling(self, state, ch=1):
        """ AC DC DCL """
        self.write(f':chAN{ch}:COUP {state}')        #Display Update State

    def Set_ChStatus(self, state, ch=1):
        """ ON OFF """
        self.write(f':chAN{ch}:STAT {state}')        #Display Update State

    def Set_ChState(self, state, ch=1):
        """ ON OFF """
        self.write(f':chAN{ch}:STAT {state}') 

    def Set_DisplayUpdate(self, state):
        """ ON OFF """
        self.write(f'SYST:DISP:UPD {state}')        #Display Update State

    def Set_System_Preset(self):
        self.write(f'SYST:PRES') 

    def Set_SweepCont(self, state):
        """ ON OFF """
        if ('ON' in state.upper()) or (state == 1):
            self.write(f'RUN')                      #Continuous
        else:
            self.write(f'SING')                     #Single

    def Set_TimeRef(self, sec):
        """ Seconds """
        self.write(f':TIM:REF {sec}')

    def Set_TimeScale(self, sec):
        """ Seconds """
        self.write(f':TIM:SCAL {sec}')

    def Set_TimeRes(self, sec):
        """ Seconds """
        self.write(f':ACQ:RES {sec}')

    def Set_VoltOffset(self, volt, ch=1):
        """ Volts """
        self.write(f':chAN{ch}:OFFS {volt}')

    def Set_VoltRange(self, volt, ch=1):
        """ Volts """
        self.write(f':chAN{ch}:RANG {volt}')

    def Set_VoltScale(self, volt, ch=1):
        """ Volts """
        self.write(f':chAN{ch}:SCAL {volt}')

###############################################################################
###  DSO Set Cursor Functions
###############################################################################
    def Set_Cursor_All_Off(self,ch=1):    
        """  1 to 4 """
        self.write(f':CURS{ch}:AOFF') 

    def Set_Cursor_State(self, state, ch=1):   
        """ ON OFF , 1 to 4 """
        self.write(f':CURS{ch}:STAT {state}') 

    def Set_Cursor_Source(self,source,ch=1):   
        """ ON OFF , Mx CxWx x = 1 to 4 """
        self.write(f':CURS{ch}:SOUR {source}') 

    def Set_Cursor_Function(self,function,ch=1):   
        """ HOR VERT PAIR ,  1 to 4 """
        self.write(f':CURS{ch}:FUNC {function}')

    def Set_Cursor_Tracking(self, state, ch=1):   
        """ ON OFF ,  1 to 4 """
        self.write(f':CURS{ch}:TRAC:STAT {state}')

    def Set_Cursor_Style(self, style,ch=1):   
        """ LINe LRHombus VLRHombus RHOMbus ,  1 to 4 """
        self.write(f':CURS{ch}:STYL {style}')

    def Set_Cursor_Max_Peak(self,ch=1):   
        """  1 to 4 """
        self.write(f':CURS{ch}:MAX:PEAK')

###############################################################################
###  DSO Set Measurement Functions
###############################################################################

    def Set_Meas_All_Off(self,ch=1):   
        """  1 to 8 """
        self.write(f':MEAS{ch}:AOFF')

    def Set_Meas_All_On(self,ch=1):   
        """  1 to 8 """
        self.write(f':MEAS{ch}:AON')

    def Set_Meas_Additional(self, mode, state, ch=1):   
        """ HIGH LOW AMP MAX MIN PDELta MEAN RMS STDD POVershoot 
            AREA RTIMe FTIMe PPULse PER FREQ PULCnt DELay ..."""
        self.write(f':MEAS{ch}:ADD {mode} {state}')

    def Set_Meas_Clear_Statistics(self, ch=1):
        """ Clear Statistics """
        self.write(f':MEAS{ch}:CLE')

    def Set_Meas_Main(self, mode, ch=1):   
        """ HIGH LOW AMP MAX MIN PDELta MEAN RMS STDD POVershoot 
            AREA RTIMe FTIMe PPULse PER FREQ PULCnt DELay ..."""
        self.write(f':MEAS{ch}:MAIN {mode}')

    def Set_Meas_Enable(self, mode, ch=1):
        """ ON OFF, ch """
        self.write(f':MEAS{ch}:ENAB {mode}')
    
    def Set_Meas_Source(self, mode, ch=1):
        """ CxWx, ch """
        self.write(f':MEAS{ch}:ENAB {mode}')

    def Set_Meas_Statistics(self, state, ch=1):   
        """ ON OFF , ch """
        self.write(f':MEAS{ch}:STAT:ENAB {state}')

###############################################################################
###  DSO Set Display Functions
###############################################################################
    def Set_Meas_Display_Position(self, mode):   
        """ PREView FLOAt DOCK """
        self.write(f':DISP:RES:DEF {mode}')


###############################################################################
### Debug Main.  Won't run when imported
###############################################################################
if __name__ == "__main__":
    DSO_Inst = DSO().jav_Open("192.168.1.100")
    DSO_Inst.jav_IDN()
    DSO_Inst.jav_ClrErr()



""" Common file functions"""
from datetime  import datetime
import os

class FileIO(object):                                       #pylint: disable=R0205
    """ Util File IO Object """
    def __init__(self):
        self.Outfile = ""
        self.sFName = ""
        self.debug = 1

    def makeFile(self,sFilepath):
        BaseDir  = os.path.dirname(os.path.realpath(sFilepath))
        BaseFile = os.path.basename(sFilepath)
        OutFile  = f'{BaseDir}\\{BaseFile}'
        self.Init(OutFile)
        return self

    def Init(self,sName="Datalog"):
        self.sFName = "%s-%s.csv"%(sName,datetime.now().strftime("%y%m%d"))
        return self

    def write(self,inStr):                                  #pylint: disable=R0201,W0613
        pass

    def write_raw(self,inStr):                              #pylint: disable=R0201,W0613
        pass

    def read(self):                                         #pylint: disable=R0201,W0613
        return 'asdkfal;sdf;jkasljk;df;jklasfd;jklasjkl;dfjkasdfljk;as;lkjf'

    def readcsv(self):                                      #pylint: disable=R0201,W0613
        return ['str1','str2','str3','str4','str5']

    def initread(self,sFile):
        self.sFName = sFile
        return self

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    FileIO = FileIO()
    FileIO.Init("FileIO.csv")
    FileIO.write("Hello World")
    FileIO.write_raw("Hello Worldd")
    data = FileIO.readcsv()
    for i, line in enumerate(data):
        print("%d:%s"%(i,",".join(data[i])))



"""Vector Signal Generator 5G NR Functions"""
from rssd.VSG.Common import VSG             #pylint: disable=E0611,E0401

class VSG(VSG):                             #pylint: disable=E0102
    """ Rohde & Schwarz Vector Signal Generator 5GNR Object """
    def __init__(self):
        super(VSG,self).__init__()          #Python2/3
        self.Model  = "SMW"
        self.sdir   = "UL"
        self.BWP    = 0
        self.User   = 0
        self.alloc  = 0
        self.subF   = 0
        self.cc     = 0         # 0 Start

    #####################################################################
    ### 5GNR Get Methods
    #####################################################################
    def Get_5GNR_BWP_CellID(self):
        rdStr = self.queryInt(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:CELL?')
        return rdStr

    def Get_5GNR_BWP_Center(self):
        rdStr = self.queryInt(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:{self.sdir}:BWP0:DFR?')
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_1stDMRSSym(self):
        #rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PUSC:DMRS:APOS?')
        rdStr = "<TBD>"
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_AddPosition(self):
        #rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PUSC:DMRS:IND?')
        if self.sdir == 'UL':
            # User/BWP-->ULBWP COnfig
            rdStr = self.query(f':SOUR:BB:NR5G:UBWP:USER0:CELL{self.cc}:UL:BWP0:PUSCH:DMTA:APIN?')
        else:
            rdStr = self.query(f':SOUR:BB:NR5G:UBWP:USER0:CELL{self.cc}:DL:BWP0:PDSCH:DMTA:APIN?')
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_Config(self):
        rdStr = "<TBD>"
        if self.sdir == 'UL':
            # User/BWP-->ULBWP COnfig
            rdStr = self.query(f':SOUR:BB:NR5G:UBWP:USER0:CELL{self.cc}:UL:BWP0:PUSCH:DMTA:CTYP?')
        else:
            rdStr = self.query(f':SOUR:BB:NR5G:UBWP:USER0:CELL{self.cc}:DL:BWP0:PDSCH:DMTA:CTYP?')
        return rdStr[1]

    def Get_5GNR_BWP_Ch_DMRS_Mapping(self):
        rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:MAPT?')
        # rdStr = "<TBD>"
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_MSymbLen(self):
        if self.sdir == 'DL':
            rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PDSCH:DMRS:LENG?')      #MMM
        else:
            rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PUSC:DMRS:LENG?')
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_RelPwr(self):
        if self.sdir == 'DL':
            rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PDSCH:DMRS:POW?')
        else:
            rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PUSCH:DMRS:POW?')
        #rdStr = "<TBD>"
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_SeqGenMeth(self):
        if self.sdir == 'DL':
            rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PDSC:DMRS:SEQG?')   #MMM
        else:
            rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PUSC:DMRS:SEQG?')
        # rdStr = "<TBD>"
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_SeqGenSeed(self):
        #Only for SeqGenMeth NICP.  Not Valid for NIDC
        if self.Get_5GNR_BWP_Ch_DMRS_SeqGenMeth() == 'DMRS':
            rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PUSC:DMRS:NSID?')
        else:
            rdStr = '<!CELL>'
        return rdStr

    def Get_5GNR_BWP_Ch_Modulation(self):
        rdStr = self.query(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:MOD?')
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_K(self):
        """ Freq Density in RB """
        if self.Get_5GNR_BWP_Ch_PTRS_State() == '1':
            rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PUSC:PTRS:FRQD?')
        else:
            rdStr = "PTRS Off"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_L(self):
        """ Time Density in OFDM Sym Freq """
        if self.Get_5GNR_BWP_Ch_PTRS_State() == '1':
            rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PUSC:PTRS:TMD?')
        else:
            rdStr = "PTRS Off"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_Pow(self):
        if self.Get_5GNR_BWP_Ch_PTRS_State() == '1':
            rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PUSC:PTRS:POW?')
        else:
            rdStr = "PTRS Off"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_RE_Offset(self):
        """ PTRS freq (RE) offset """
        if self.Get_5GNR_BWP_Ch_PTRS_State() == '1':
            rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PUSC:PTRS:REOF?')
        else:
            rdStr = "PTRS Off"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_State(self):
        rdStr = self.query(f':SOUR:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:PUSC:PTRS:STAT?')
        return rdStr

    def Get_5GNR_BWP_Ch_ResBlock(self):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        rdStr = self.query(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:RBN?')
        return rdStr

    def Get_5GNR_BWP_Ch_ResBlockOffset(self):
        rdStr = self.query(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:RBOF?')
        return rdStr

    def Get_5GNR_BWP_Ch_SymbNum(self):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        rdStr = self.query(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:SYMN?')
        return rdStr

    def Get_5GNR_BWP_Ch_SymbOff(self):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        rdStr = self.query(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:SYM?')
        return rdStr

    def Get_5GNR_BWP_Count(self):
        rdStr = self.query(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:{self.sdir}:NBWP?')
        return rdStr

    def Get_5GNR_BWP_ResBlock(self):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        rdStr = self.query(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:{self.sdir}:BWP0:RBN?')
        return rdStr

    def Get_5GNR_BWP_ResBlockOffset(self):
        rdStr = self.query(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:{self.sdir}:BWP0:RBOF?')
        return rdStr

    def Get_5GNR_BWP_SlotNum(self):
        ### Number of slots
        rdStr = self.query(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:SLOT?')
        return rdStr

    def Get_5GNR_BWP_SubSpace(self):
        rdStr = self.query(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:{self.sdir}:BWP0:SCSP?')
        return rdStr

    def Get_5GNR_CC_Freq(self):
        offset = self.Get_5GNR_CC_Offset()
        freq    = self.Get_Freq()
        return offset + freq

    def Get_5GNR_CC_Offset(self):
        rdStr = self.queryFloat(f'SOUR1:BB:NR5G:NODE:CARM:DFREQ:ROW{self.cc}?')
        return rdStr

    def Get_5GNR_SSB_SubSpace(self):
        if self.sdir == 'DL':
            rdStr = self.query(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:SSPB0:SCSP?')
        else:
            rdStr = '<UL n/a>'
        return rdStr

    def Get_5GNR_BWP_SubSpaceTotal(self):
        rdStr = []
        rdStr.append(self.query(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S15K:NRB?'))
        rdStr.append(self.query(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S30K:NRB?'))
        rdStr.append(self.query(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S60K:NRB?'))
        rdStr.append(self.query(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S120K:NRB?'))
        return rdStr

    def Get_5GNR_ChannelBW(self):
        ### 5;10;15;20;25;30;40;50;60;70;80;90;100;200;400
        rdStr = self.query(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:CBW?')
        return rdStr

    def Get_5GNR_Direction(self):
        rdStr = self.query(f':SOUR1:BB:NR5G:LINK?')
        if rdStr == 'DOWN':
            self.sdir = "DL"
            self.alloc = 1         #Alloc 0:CORSET
        elif rdStr == 'UP':
            self.sdir = "UL"
            self.alloc = 0         #Alloc 0:PUSCH
        else:
            print('Get_5GNR_Direction Error')
        return self.sdir

    def Get_5GNR_FreqRange(self):
        rdStr = self.query(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:CARD?')
        if (rdStr == 'LT3') or (rdStr == 'LT3') or (rdStr == 'FR1LT3'):
            outStr = 'LOW'
        elif (rdStr == 'BT36') or (rdStr == 'BT37125')or (rdStr == 'FR1GT3'):
            outStr = 'MIDD'
        elif (rdStr == 'GT6') or (rdStr == 'GT7125')or (rdStr == 'FR2'):
            outStr = 'HIGH'
        else:
            outStr = 'Error Get_5GNR_FreqRange {rdStr}'
        return outStr

    def Get_5GNR_RefA(self):
        rdStr = self.queryInt(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:POIN?')
        return rdStr

    def Get_5GNR_RBMax(self):
        odata = []
        rdStr = self.queryInt(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S15K:NRB?')
        odata.append([15,rdStr])
        rdStr = self.queryInt(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S30K:NRB?')
        odata.append([30,rdStr])
        rdStr = self.queryInt(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S60K:NRB?')
        odata.append([60,rdStr])
        rdStr = self.queryInt(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S120K:NRB?')
        odata.append([120,rdStr])
        return odata

    def Get_5GNR_PhaseCompensate(self):
        rdStr = self.query(f':SOUR:BB:NR5G:NODE:RFPH:STAT?')
        return rdStr

    def Get_5GNR_PhaseCompensate_Freq(self):
        rdStr = self.query(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:PCFR?')
        return rdStr

    def Get_5GNR_TM_Cat(self):
        rdStr = self.query(f'SOUR1:BB:NR5G:SETT:TMOD:DL:CAT?').split(',')
        return rdStr

    def Get_5GNR_TransPrecoding(self):
        # SC-FDMA or DFT-S-OFDM
        # 5GNR--> User/BWP --> UL BWP Config --> PUSCH --> TP
        # rdStr = self.query(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL0:TPST?')
        rdStr = self.query(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:UL:BWP0:PUSC:TPST?')   #4.50
        return rdStr

    #####################################################################
    ### 5GNR Settings
    #####################################################################
    def Set_5GNR_BBState(self,iEnable):
        """ON OFF"""
        if (iEnable == 1) or (iEnable == 'ON'):
            self.jav_Wait(':SOUR1:BB:NR5G:STAT 1')
#            self.query(f'*OPC?')          # Wait for calculation
        else:
            self.write(f':SOUR1:BB:NR5G:STAT 0')

    def Set_5GNR_BWP_CellID(self, iCellID):
        self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:CELL {iCellID}')

    def Set_5GNR_BWP_Ch_Modulation(self,sMod):
        self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:MOD {sMod}')

    def Set_5GNR_BWP_Ch_ResBlock(self,iRB):
        """5GNR-->Scheduling-->PUSCH-->No. RBs"""
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:RBN {iRB}')

    def Set_5GNR_BWP_Ch_ResBlockOffset(self,iRBO):
        """5GNR-->Scheduling-->PUSCH-->RB Offset"""
        self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:RBOF {iRBO}')

    def Set_5GNR_BWP_Corset_ResBlock(self, iRB):
        if self.sdir == 'DL':
            self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL0:RBN {iRB}')

    def Set_5GNR_BWP_Corset_ResBlockOffset(self,iRBO):
        if self.sdir == 'DL':
        ### 5GNR-->Scheduling-->PUSCH-->No. RBs
            self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL0:RBOF {iRBO}')

    def Set_5GNR_BWP_ResBlock(self,iRB):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        self.write(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:{self.sdir}:BWP0:RBN {iRB}')

    def Set_5GNR_BWP_ResBlockMax(self):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        MaxRB = 20
        rdStr = self.query(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:{self.sdir}:BWP0:RBN {MaxRB}')
        return rdStr

    def Set_5GNR_BWP_ResBlockOffset(self,iRBO):
        self.write(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:{self.sdir}:BWP0:RBOF {iRBO}')

    def Set_5GNR_BWP_SubSpace(self,iSubSp):
        """15 30 60 120"""
        if iSubSp == 15:
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S15K:USE 1')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S30K:USE 0')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S60K:USE 0')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S120K:USE 0')
        elif iSubSp == 30:
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S30K:USE 1')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S15K:USE 0')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S60K:USE 0')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S120K:USE 0')
        elif iSubSp == 60:
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S60K:USE 1')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S15K:USE 0')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S30K:USE 0')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S120K:USE 0')
        elif iSubSp == 120:
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S120K:USE 1')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S15K:USE 0')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S30K:USE 0')
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:S60K:USE 0')
        else:
            print('Subcarrier spacing not supported')
        self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:TXBW:RES')
        self.write(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:{self.sdir}:BWP0:SCSP N{iSubSp}')

    def Set_5GNR_BWP_Users(self,iUsers):
        """Number of users"""
        self.write(f':SOUR1:BB:NR5G:UBWP:NUS {iUsers}')

    def Set_5GNR_CC_Num(self,iCC):
        """ iCC, 1 start """
        self.cc = iCC - 1
        self.write(f'SOUR1:BB:NR5G:NODE:NCAR {iCC}')

    def Set_5GNR_CC_Offset(self,Freq):
        """ freq, Hz """
        self.write(f':SOUR1:BB:NR5G:NODE:CARM:DFR:ROW{self.cc} {Freq}')

    def Set_5GNR_ChannelBW(self,iBW):
        """ BW in MHz
         5GNR-->NODE-->Carriers-->Channel BW"""
        self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:CBW BW{iBW}')

    def Set_5GNR_Direction(self,sDirection):
        """ UP| DOWN """
        if (sDirection == "UL") or (sDirection == "UP"):
            self.write(f':SOUR1:BB:NR5G:LINK UP')
            self.sdir = "UL"
            self.alloc = 0         #Alloc 0:PUSCH
        elif (sDirection == "DL") or (sDirection == "DOWN"):
            self.write(f':SOUR1:BB:NR5G:LINK DOWN')
            self.sdir = "DL"
            self.alloc = 1         #Alloc 0:Coreset 1:PDSCH
        else:
            print("Set_5GNR_Direction must be UP or DOWN")

    def Set_5GNR_FRC_State(self,state):
        """ 'ON' | 'OFF' """
        if (state == "ON") or (state == 1):
            self.write(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:UL:BWP0:FRC:STAT ON')
        else:
            self.write(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:UL:BWP0:FRC:STAT OFF')

    def Set_5GNR_FreqRange(self,iRange):
        """ 0:<3GHz 1:3-6GHz 2:>6GHz """
        # """ LOW; MIDD; HIGH """
        if (iRange==0) or (iRange == 'LOW'):
            # self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:CARD LT3')     #4.70.026.51
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:CARD FR1LT3')    #C45.4.70.026.51.131
        elif (iRange==1) or (iRange == 'MIDD'):
            # self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:CARD BT36')    #4.70.026.51
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:CARD FR1GT3')    #C45.4.70.026.51.131
        elif (iRange==2) or (iRange == 'HIGH'):
            # self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:CARD GT6')     #4.70.026.51
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:CARD FR2')       #C45.4.70.026.51.131

    def Set_5GNR_GenerateWv(self,sName):
        """ Generate Waveform File"""
        self.write(f':SOUR1:BB:NR5G:WAV:CRE "{sName}"')
        self.delay(2)
        self.jav_OPC_Wait('*IDN?')

    def Set_5GNR_Parameters(self,sDir):
        self.Set_5GNR_Direction(sDir)

    def Set_5GNR_PhaseCompensate(self,state):
        """ 'ON' | 'OFF' """
        if (state == "ON") or (state == 1):
            self.write(f':SOUR:BB:NR5G:NODE:RFPH:MODE AUTO')
        else:
            self.write(f':SOUR:BB:NR5G:NODE:RFPH:MODE OFF')

    def Set_5GNR_PhaseCompensate_Freq(self,Freq):
        self.write(f':SOUR:BB:NR5G:NODE:CELL{self.cc}:PCFR {Freq}')

    def Set_5GNR_SSB(self):
        """Num DL SS/PBCH Patterns"""
        if self.sdir == 'DL':
            self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:NSSP 1')

    def Set_5GNR_TM(self, file):
        """NR-FR1-TM1_1__FDD_100MHz_30kHz """
        self.query(f'SOUR1:BB:NR5G:SETT:TMOD:DL "{file}";*OPC?')

    def Set_5GNR_TransPrecoding(self, sState):
        """ SC-FDMA or DFT-S-OFDM
        5GNR--> User/BWP --> UL BWP Config --> PUSCH --> TP  """
        if sState == 'ON':
            # self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL0:TPST ON') #4.30SP2?
            self.write(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:UL:BWP0:PUSC:TPST ON')     #4.50
        else:
            # self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL0:TPST OFF') #4.30SP2?
            self.write(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:UL:BWP0:PUSC:TPST OFF')    #4.50

    def Set_5GNR_Setting_Load(self, sName):
        self.query(f':SOUR:BB:NR5G:SETT:LOAD "/var/user/{sName}";*OPC?')

    def Set_5GNR_savesetting(self, sName):
        self.query(f':SOUR:BB:NR5G:SETT:STOR "/var/user/{sName}";*OPC?')

    def Set_5GNR_saveAllocation(self, sName):
        self.query(f'SOUR1:BB:NR5G:ANAL:CONT "/var/user/{sName}";*OPC?')

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    # this won't be run when imported
    SMW = VSG()
    SMW.jav_Open("192.168.1.114")
    SMW.cc = 1
    print(SMW.Get_5GNR_CC_Freq())
    SMW.jav_Close()



"""Purpose: rssd.self.VSE.common.py driver test
"""
host = '127.0.0.1'                              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSE.K96 import VSE

class TestGeneral(unittest.TestCase):
    def setUp(self):                            #run before each test
        self.VSE = VSE().jav_OpenTest(host)

    def tearDown(self):                         #Run after each test
        self.assertEqual(self.VSE.jav_Error()[0],'0')
        self.VSE.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_VSE_Example(self):
        Fs = 122.88e6
        IQFile = 'IQFile.iqw'
        OFDMCfg = "\\misc\\OFDM.xml"
        self.VSE.jav_Reset()
        self.VSE.Init_K96()                          #Change Channel
        self.VSE.Set_DisplayUpdate("ON")             #Display On
        self.VSE.Set_SweepCont(0)                    #Continuous Sweep Off
        self.VSE.Set_IQ_SamplingRate(Fs)             #Sampling Rate
        self.VSE.Set_File_InputIQW(Fs,IQFile)        #VSE Input File
        self.VSE.Set_K96_File_Config(OFDMCfg)        #K96 Demod File
        self.VSE.Set_K96_BurstSearch("OFF")          #Burst Search off
        self.VSE.Set_K96_OFDMSymbols(14)
        self.VSE.K96_EVM_AutoCal()
        getVal = self.VSE.Get_EVM()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
#coverage run -a -m unittest -b -v test_HW_VSE_Common
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=4).run(suite)



### Reference: https://github.com/Terrabits/rohdeschwarz/blob/master/setup.py
### Reference: https://python-packaging.readthedocs.io/en/latest/minimal.html
###
### python setup.py register #Reserve name in pypi
### python setup.py --help-commands #Help
### python setup.py bdist           #Creates <pkg>.zip
### python setup.py install         #Installs package
### python setup.py install_scripts
### pip install .                   #Installs package in directory
### pip install -e .                #Install editable package
##########################################################
### Upload to PyPi
### python setup.py sdist           #Creates <pkg>.tar.gz
### twine upload .\dist\rssd-0.1.8.tar.gz

# import os
from setuptools import setup, find_packages

with open('README.md') as f:
    long_description = f.read()

setup(
    name='rssd',
    version='2021.12.22',
    description='Rohde & Schwarz SCPI Driver',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 4 - Beta',      #3:Alpha 4:Beta 5:Production/Stable
        'License :: Freely Distributable',
        'License :: Other/Proprietary License',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)',
        'Topic :: System :: Hardware :: Hardware Drivers',
    ],
    keywords='Rohde Schwarz test equipment FSW FSV NRP NRQ OSP SGT SMA SMB SMBV SMW SCPI VSA VSG VST',
    url='https://github.com/mclim9/rssd',
    author='Martin Lim',
    author_email='martin.lim@rsa.rohde-schwarz.com',
    license='R&S Terms and Conditions for Royalty-Free Products',
    packages=find_packages(exclude=['test','proto']),
    #packages=['rssd'],
    install_requires=[
        'pyvisa>=1.9.0',
        'future_fstrings>=1.0.0',
    ],
    test_suite = 'test',
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'rssd=rssd.bin.cli:main'
        ],
    },
)



"""IQ --> SCPI --> SMW"""
SMW_IP   = '192.168.1.114'

###############################################################################
### Code Overhead: Import and create objects
###############################################################################
import numpy            as np
from rssd.VSG.Common    import VSG
SMW = VSG().jav_Open(SMW_IP)  #Create SMW Object

###############################################################################
### Code Start
###############################################################################
### :MMEM:DATA:UNPR "NVWFM://var//user//<wave.wv>",#<numSize><NumBytes><I0Q0...IxQx>
###     wave.wv : Name of *.wv to be created
###     numSize : Number of bytes in <NumBytes> string
###     NumBytes: Number of bytes to follow
###               Each I (or Q) value is two bytes
###               I(2 bytes) + Q(2bytes) = 4 bytes/IQ pair
###               NumBytes = NumIQPair * 4
###
IData = [0.1,0.2,0.3]
QData = [0.4,0.5,0.6]

### ASCII
scpi  = ':MMEM:DATA:UNPR "NVWFM://var//user//test.wv",#'        # Define file & number of bytes
iqsize= str(len(IData)*4)                                       # Calculate bytes of IQ data
scpi  = scpi + str(len(iqsize)) + iqsize                        # Calculate length of iqsize string

### Binary: IQ data --> Binary format
iqdata= np.vstack((IData,QData)).reshape((-1,),order='F')       # Combine I&Q Data
bits  = np.array(iqdata*32767, dtype='>i2')                     # Convert to big-endian 2byte int

### ASCII + Binary
cmd   = bytes(scpi, 'utf-8') + bits.tostring()                  # Add ASCII + Bin
SMW.K2.write_raw(cmd)
SMW.write('SOUR1:BB:ARB:WAV:CLOC "/var/user/test.wv",1.234E6')  # Set Fs/Clk Rate
SMW.write('BB:ARB:WAV:SEL "/var/user/test.wv"')                 # Select Arb File
print(SMW.query('SYST:ERR?'))

###############################################################################
### Close Nicely
###############################################################################
SMW.jav_Close()



"""Vector Signal Analyzer 5GNR Functions"""
#pylint: disable=E0611,E0401
from rssd.VSA.Common import VSA

class VSA(VSA):                                 #pylint: disable=E0102
    """ Rohde & Schwarz Vector Signal Analyzer 5GNR Object """
    def __init__(self):
        super(VSA, self).__init__()             #Python 2/3
        self.sdir   = "UL"
        self.cc     = 1                         #1 Start
        self.subF   = 1
        self.alloc  = 1

    #####################################################################
    ### FSW 5GNR Get
    #####################################################################
    def Get_5GNR_ACLR(self):
        rdStr = self.query(':CALC:MARK:FUNC:POW:RES? MCAC')
        return rdStr

    def Get_5GNR_BWP_Center(self):
        SS = self.Get_5GNR_BWP_SubSpace()
        SS = int(''.join(c for c in SS if c.isdigit()))
        RB = int(self.Get_5GNR_BWP_ResBlock())
        RBO = int(self.Get_5GNR_BWP_ResBlockOffset())
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:RBC MAX')
        RBMax = int(self.Get_5GNR_BWP_ResBlock())
        self.Set_5GNR_BWP_ResBlock(RB)
        self.Set_5GNR_BWP_ResBlockOffset(RBO)
        ressy = (SS * 1e3 *12*(RB - RBMax + 2 * RBO))/2
        return ressy

    #####################################################################
    ### FSW 5GNR DMRS Config
    #####################################################################
    def Get_5GNR_BWP_Ch_DMRS_1stDMRSSym(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:TAP?')
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_AddPosition(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:MSYM:APOS?')
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_Config(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:CTYP?')
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_MSymbLen(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:MSYM:LENG?')
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_Mapping(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:MTYP?')
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_RelPwr(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:POW?')
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_SeqGenMeth(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:SGEN?')
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_SeqGenSeed(self):
        #Only for SeqGenMeth NICP.  Not Valid for NIDC
        if self.Get_5GNR_BWP_Ch_DMRS_SeqGenMeth() == 'NIDP':
            rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:NID?')
        else:
            rdStr = '<!CELL>'
        return rdStr

    def Get_5GNR_BWP_Ch_Modulation(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:MOD?')
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_K(self):
        """ Freq Density in RB """
        if self.Get_5GNR_BWP_Ch_PTRS_State() == '1':
            rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:PTRS:K?')
        else:
            rdStr = "PTRS Off"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_L(self):
        """ Time Density in OFDM Sym Freq """
        if self.Get_5GNR_BWP_Ch_PTRS_State() == '1':
            rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:PTRS:L?')
        else:
            rdStr = "PTRS Off"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_Pow(self):
        if self.Get_5GNR_BWP_Ch_PTRS_State() == '1':
            rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:PTRS:POW?')
        else:
            rdStr = "PTRS Off"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_RE_Offset(self):
        """ PTRS freq (RE) offset """
        if self.Get_5GNR_BWP_Ch_PTRS_State() == '1':
            rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:PTRS:REOF?')
        else:
            rdStr = "PTRS Off"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_State(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:PTRS:STAT?')
        return rdStr

    def Get_5GNR_BWP_Ch_ResBlock(self):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:RBC?')
        return rdStr

    def Get_5GNR_BWP_Ch_ResBlockOffset(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:RBOF?')
        return rdStr

    def Get_5GNR_BWP_Ch_SymbNum(self):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:SCO?')
        return rdStr

    def Get_5GNR_BWP_Ch_SymbOff(self):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:SOFF?')
        return rdStr

    def Get_5GNR_BWP_Count(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWPC?')
        return rdStr

    def Get_5GNR_BWP_ResBlock(self):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:RBC?')
        return rdStr

    def Get_5GNR_BWP_ResBlockOffset(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:RBOF?')
        return rdStr

    def Get_5GNR_BWP_SlotNum(self):
        ### Number of slots
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SCO?')
        return rdStr

    def Get_5GNR_BWP_SubSpace(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SSP?')
        return rdStr

    def Get_5GNR_CC_Freq(self):
        rdStr = self.queryFloat(f'SENS:FREQ:CENT:CC{self.cc}?')
        return rdStr

    def Get_5GNR_CC_Offset(self):
        rdStr = self.queryFloat(f'SENS:FREQ:CENT:CC{self.cc}:OFFS?')
        return rdStr

    def Get_5GNR_ChPwr(self):
        Power = self.queryFloat(f'FETC:CC{self.cc}:ISRC:FRAM:SUMM:POW:AVER?')
        return Power

    def Get_5GNR_ChannelBW(self):
        ### 5;10;15;20;25;30;40;50;60;70;80;90;100;200;400
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:BW?')
        return rdStr

    def Get_5GNR_CrestFactor(self):
        rdStr = self.queryFloat(f':FETC:CC{self.cc}:ISRC:SUMM:CRES:AVER?')
        return rdStr

    def Get_5GNR_Direction(self):
        rdStr = self.query(f':CONF:NR5G:LDIR?')
        if rdStr == 'DL':
            self.sdir = "DL"
            self.alloc = 1         #Alloc 0:CORSET
        elif rdStr == 'UL':
            self.sdir = "UL"
            self.alloc = 0         #Alloc 0:PUSCH
        else:
            print('Get_5GNR_Direction Error')
        return rdStr

    def Get_5GNR_EVM_All(self):
        EVM = self.queryFloat(f':FETC:CC{self.cc}:SUMM:EVM:ALL:AVER?')
        return EVM

    def Get_5GNR_EVM_PhysCh(self):
        EVM = self.queryFloat(f':FETC:CC{self.cc}:SUMM:EVM:PCH:AVER?')
        return EVM

    def Get_5GNR_EVM_PhysSig(self):
        EVM = self.queryFloat(f':FETC:CC{self.cc}:SUMM:EVM:PSIG:AVER?')
        return EVM

    def Get_5GNR_Params_EVM(self,header=0):
        if header != 1:
            Crest   = self.Get_5GNR_CrestFactor()
            Power   = self.Get_5GNR_ChPwr()
            EVMAll  = self.Get_5GNR_EVM_All()
            EVMPhyC = self.Get_5GNR_EVM_PhysCh()
            EVMPhyS = self.Get_5GNR_EVM_PhysSig()
            outStr  = f"{Crest:6.3f},{Power:6.3f},{EVMAll:.2f},{EVMPhyC:.2f},{EVMPhyS:.2f}"
        else:
            outStr  = 'K144_Cres,K144Pwr,5GEVM_All,EVM_PhyCh(PxSCH),EVM_PhySig(DMRS)'
        return outStr

    def Get_5GNR_FreqRange(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:DFR?')
        return rdStr

    def Get_5GNR_Meas_ACLR(self):
        rdStr = self.query(f':CALC:MARK:FUNC:POW:RES? MCAC')
        return rdStr

    def Get_5GNR_RefA(self):
        rdStr = self.queryInt(f':CONF:NR5G:{self.sdir}:CC{self.cc}:RPA:RTCF?')
        return rdStr

    def Get_5GNR_PhaseCompensate(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:RFUC:STAT?')
        return rdStr

    def Get_5GNR_PhaseCompensate_Freq(self):
        rdStr = self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:RFUC:FZER:FREQ?')
        return rdStr

    def Get_5GNR_SEM(self):
        rdStr = self.query(f':CALC1:LIM:FAIL?')
        return rdStr

    def Get_5GNR_SSB_SubSpace(self):
        if self.sdir == 'DL':
            SyncDetStat = self.query(f':CONF:NR5G:DL:SSBL1:DET?')
            if SyncDetStat == 'MAN':
                rdStr = self.query(f':CONF:NR5G:DL:CC{self.cc}:SSBL1:SSP?')
            else:
                rdStr = '<Scanng>'
        else:
            rdStr = '<UL n/a>'
        return rdStr

    def Get_5GNR_TM_Cat(self):
        rdStr = 'Command not available'
        # rdStr = self.query(f'MMEM:LOAD:TMOD:CC{self.cc}:CAT?').split(',')
        return rdStr

    def Get_5GNR_TransPrecoding(self):
        """UL Only"""
        if self.sdir == 'UL':
            rdStr = self.query(f':CONF:NR5G:UL:CC{self.cc}:TPR?')
        else:
            rdStr = '<DL N/A>'
        return rdStr

    #####################################################################
    ### FSW 5GNR Init
    #####################################################################
    def Init_5GNR(self):
        self.Set_Channel('NR5G')
        self.write(f'CONF:NR5G:MEAS EVM')

    def Init_5GNR_Meas(self,sMeas):
        """ EVM; ESPectrum; ACLR; TAER"""
        self.write(f'CONF:NR5G:MEAS {sMeas}')

    def Init_5GNR_SEM(self):
        self.Set_Channel('NR5G')
        self.write(':CONF:NR5G:MEAs ESP')

    #####################################################################
    ### FSW 5GNR Settings
    #####################################################################
    def Set_5GNR_AutoEVM(self):
        # self.jav_OPC_Wait(':SENS:ADJ:EVM')
        self.jav_Wait(':SENS:ADJ:EVM')

    def Set_5GNR_AutoRefLvl(self):
        pass

    def Set_5GNR_AllocFile(self,sFilename):
        # \Instr\user\V5GTF\AllocationFiles\UL
        self.jav_OPC_Wait(f'MMEM:LOAD:DEM:CC{self.cc} "{sFilename}"')

    def Set_5GNR_AllocFileSave(self,sFilename):
        self.write(f'MMEM:STOR:DEM:CC{self.cc} "C:\\R_S\\Instr\\user\\{sFilename}.allocation"')

    def Set_5GNR_BWP_Ch_DMRS_1stDMRSSym(self,DMRS):
        """ 2 3 """
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:TAP {DMRS}')

    def Set_5GNR_BWP_Ch_DMRS_AddPosition(self,DMRS):
        """ 0 1 2 3 """
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:MSYM:APOS {DMRS}')

    def Set_5GNR_BWP_Ch_DMRS_Config(self,DMRS):
        """ 1 or 2 """
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:CTYP {DMRS}')

    def Set_5GNR_BWP_Ch_DMRS_MSymbLen(self,DMRS):
        """ 1 2 """
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:MSYM:LENG {DMRS}')

    def Set_5GNR_BWP_Ch_DMRS_Mapping(self,DMRS):
        """ A B """
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:MTYP {DMRS}')

    def Set_5GNR_BWP_Ch_DMRS_RelPwr(self,DMRS):
        """ -25 to 12 dB """
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:POW {DMRS}')

    def Set_5GNR_BWP_Ch_DMRS_SeqGenMeth(self,DMRS):
        """ NIDC NIDD """
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:SGEN {DMRS}')

    def Set_5GNR_BWP_Ch_DMRS_SeqGenSeed(self,DMRS):
        """ 0 1007 """
        #Only for SeqGenMeth NIDD.  Not Valid for NIDC
        SeqGenMeth = self.Get_5GNR_BWP_Ch_DMRS_SeqGenMeth()
        if SeqGenMeth == 'NIDD':
            self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:DMRS:NID {DMRS}')
        elif SeqGenMeth == 'NIDC':
            pass
        else:
            print('Get_5GNR_BWP_Ch_DMRS_SeqGenSeed Method?')

    def Set_5GNR_BWP_Ch_Modulation(self,sMod):
        """QPSK; QAM16; QAM64; QAM256; PITB"""
        if self.sdir == 'DL':
            self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:CW:MOD {sMod}')
        else:
            self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:MOD {sMod}')

    def Set_5GNR_BWP_Ch_ResBlock(self,iRB):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:RBC {iRB}')

    def Set_5GNR_BWP_Ch_ResBlockOffset(self,iRBO):
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:RBOF {iRBO}')

    def Set_5GNR_BWP_Corset_ResBlock(self, iRB):
        if self.sdir == 'DL':
            self.write(f':CONF:NR5G:DL:CC{self.cc}:FRAM1:BWP0:SLOT0:COR0:RBC {iRB}')

    def Set_5GNR_BWP_Corset_ResBlockOffset(self,iRBO):
        if self.sdir == 'DL':
            self.write(f':CONF:NR5G:DL:CC{self.cc}:FRAM1:BWP0:SLOT0:COR0:RBOF {iRBO}')

    def Set_5GNR_BWP_ResBlock(self,iRB):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:RBC {iRB}')

    def Set_5GNR_BWP_ResBlockOffset(self,iRBO):
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:RBOF {iRBO}')

    def Set_5GNR_BWP_Ch_SymbNum(self, symb):
        """Set BWP Ch Symbol Offset"""
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:SCO {symb}')

    def Set_5GNR_BWP_Ch_SymbOff(self, symb):
        """Set BWP Ch Symbol Offset"""
        self.query(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SLOT0:ALL0:SOFF {symb}')

    def Set_5GNR_BWP_SubSpace(self,iSubSp):
        """15 30 60 120"""
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:FRAM1:BWP0:SSP SS{iSubSp}')

    def Set_5GNR_CC_Num(self,iNumCC):
        self.write(f'CONF:NR5G:NOCC {iNumCC}')

    def Set_5GNR_CC_Offset(self,iCC, fFreq):
        if iCC > 1:
            self.write(f':SENS:FREQ:CENT:CC{iCC}:OFFS {fFreq}')

    def Set_5GNR_CC_Capture(self,sCapture):
        """SING AUTO"""
        if self.cc > 1:
            self.write(f':CONF:NR5G:CSC {sCapture}')
        else:
            pass

    def Set_5GNR_CellID(self,iCellID):
        self.write(f'CONF:NR5G:{self.sdir}:CC{self.cc}:PLC:CID {iCellID}')

    def Set_5GNR_ChannelBW(self,iBW):
        """5;10;15;20;25;30;40;50;60;70;80;90;100;200;400"""
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:BW BW{iBW}')

    def Set_5GNR_Direction(self,sDirection):
        """UL | DL"""
        if (sDirection == "UL") or (sDirection == "UP"):
            self.write(':CONF:NR5G:LDIR UL')
            if self.Get_5GNR_TransPrecoding() == '0':
                self.write(f':CONF:NR5G:UL:CC{self.cc}:IDC ON')
            self.write(f':CONF:NR5G:UL:CC{self.cc}:FRAM1:BWP0:SLOT0:FORM 1')     #All UL
            self.sdir = "UL"
        elif (sDirection == "DL") or (sDirection == "DOWN"):
            self.write(f':CONF:NR5G:LDIR DL')
            self.write(f':CONF:NR5G:DL:CC{self.cc}:FRAM1:BWP0:SLOT0:FORM 0')     #All DL
            self.write(f':CONF:NR5G:DL:CC{self.cc}:IDC ON')
            self.sdir = "DL"
        else:
            if self.debug: print("Set_5GNR_UL_Direction must be UL or DL")

    def Set_5GNR_EVMUnit(self,sUnit):
        """ DB | PCT """
        self.write(f'UNIT:EVM {sUnit}')

    def Set_5GNR_FrameCount(self,State):
        """ON OFF"""
        if (State=='ON') or (State == 1):
            self.write(f':SENS:NR5G:FRAM:COUN:STAT ON')
        elif(State=='OFF') or (State == 0):
            self.write(f':SENS:NR5G:FRAM:COUN:STAT OFF')
        else:
            pass

    def Set_5GNR_FreqRange(self,iRange):
        """ 0:<3GHz 1:3-6GHz 2:>6GHz """
        ### LOW; MIDD; HIGH
        if (iRange==0) or (iRange == 'LOW'):
            self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:DFR LOW')
        elif (iRange==1) or (iRange == 'MIDD'):
            self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:DFR MIDD')
        elif (iRange==2) or (iRange == 'HIGH'):
            self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:DFR HIGH')
        else:
            print('Set_5GNR_FreqRange invalid parameter')

    def Set_5GNR_PhaseCompensate(self,state):
        """ 'ON' | 'OFF' """
        if (state == "ON") or (state == 1):
            self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:RFUC:STAT ON')
        else:
            self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:RFUC:STAT OFF')

    def Set_5GNR_PhaseCompensate_Freq(self,Freq):
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:RFUC:FZER:MODE MAN')
        self.write(f':CONF:NR5G:{self.sdir}:CC{self.cc}:RFUC:FZER:FREQ {Freq}')

    def Set_5GNR_Result_View(self, sMode):
        """ALL | VIEW """
        if self.cc > 1:
            self.query(f':SENS:NR5G:RSUM:CCR {sMode}')

    def Set_5GNR_savesetting(self, sName):
        self.query(f":MMEM:STOR:DEM:CC{self.cc} 'C:\\R_S\\Instr\\user\\NR5G\\AllocationFiles\\{sName}.allocation';*OPC?")

    def Set_5GNR_SEM_Freq(self,fFreq,dSubBlock=1):
        self.write(f':SENS:ESP{dSubBlock}:SCEN {fFreq}')

    def Set_5GNR_SEM_SubBlockNum(self,dSubBlock):
        self.write(f':SENS:ESP:SCO {dSubBlock}')

    def Set_5GNR_SubFrameCount(self,dSubFrame):
        self.write(f':SENS:NR5G:FRAM:COUN:STAT OFF')
        self.write(f':SENS:NR5G:FRAM:SLOT {dSubFrame}')

    def Set_5GNR_TM(self, file):
        """NR-FR1-TM1_1__FDD_15MHz_30kHz"""
        self.query(f'MMEM:LOAD:TMOD:CC{self.cc} "{file}";*OPC?')

    def Set_5GNR_TransPrecoding(self,sState):
        """UL only: ON | OFF """
        if self.sdir == 'UL':
            self.write(f':CONF:NR5G:UL:CC{self.cc}:TPR {sState}')
        else:
            print('<DL TransPrecoding N/A>')

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    FSW = VSA().jav_Open("192.168.1.109")
    FSW.cc = 1
    FSW.Set_5GNR_AutoEVM()
    print(FSW.Get_5GNR_Params_EVM())
    FSW.jav_ClrErr()
    FSW.jav_Close()



"""Vector Signal Analyzer LTE Functions"""
from datetime           import datetime     #pylint: disable=E0611,E0401
from rssd.VSA.Common    import VSA          #pylint: disable=E0611,E0401

class VSA(VSA):                        #pylint: disable=E0102
    """ Rohde & Schwarz Vector Signal Analyzer LTE Object
        K100 3GPP LTE FDD Downlink
        K101 3GPP LTE FDD Uplink
        K102 3GPP LTE MIMO Downlink
        K103 3GPP LTE MIMO Uplink
        K104 3GPP LTE TDD Downlink
        K105 3GPP LTE TDD Uplink
        K106 3GPP LTE NB-IoT TDD Downlink
        """
    def __init__(self):
        super(VSA, self).__init__()
        self.ldir = "DL"
        self.cc   = 1

    #####################################################################
    ### VSA Query
    #####################################################################
    def Get_ACLR(self):
        ACLR = self.queryFloatArry(':CALC:MARK:FUNC:POW:RES? MCAC')
        return ACLR

    def Get_EVM_n_Params(self):
        MAttn   = self.Get_AttnMech()
        RefLvl  = self.Get_RefLevel()
        Power   = self.Get_ChPwr()
        EVM     = self.Get_EVM()
        return "%.2f,%.2f,%6.2f,%.2f"%(MAttn,RefLvl,Power,EVM)

    def Get_LTE_CC(self):
        rdStr = self.query(':CONF:LTE:NOCC?')
        return rdStr

    def Get_LTE_ChBW(self):
        rdStr = self.query(f':CONF:LTE:{self.ldir}:BW?')
        return rdStr

    def Get_LTE_ChPwr(self):
        rdStr = self.queryFloat('FETC:CC1:SUMM:POW:AVER?')
        return rdStr

    def Get_LTE_CrestFactor(self):
        rdStr = self.queryFloat('FETC:CC1:SUMM:CRES:AVER?')
        return rdStr

    def Get_LTE_Direction(self):
        rdStr = self.query(':CONF:LDIR?')
        if rdStr == 'DL':
            self.ldir = "DL"
        elif rdStr == 'UL':
            self.ldir = "UL"
        else:
            if self.debug: print('Get_LTE_Direction Error')
        # print(f'LTE DIRECTION = {self.ldir}')
        return rdStr

    def Get_LTE_Duplex(self):
        rdStr = self.query(':CONF:LTE:DUPL?')
        return rdStr

    def Get_LTE_EVM(self):
        rdStr = self.queryFloat('FETC:SUMM:EVM?')
        return rdStr

    def Get_LTE_EVMParams(self):
        Crest = self.Get_LTE_CrestFactor()
        Power = self.Get_LTE_ChPwr()
        EVM   = self.Get_LTE_EVM()
        return f"{Crest:6.3f},{Power:6.3f},{EVM:.2f}"

    def Get_LTE_Modulation(self):
        if self.ldir == 'UL':
            rdStr = self.query(f':CONF:LTE:{self.ldir}:SUBF0:ALL:MOD?')
        elif self.ldir == 'DL':
            rdStr = self.query(f':CONF:LTE:{self.ldir}:SUBF0:ALL0:MOD?')
        else:
            print('LDIR error in Get_LTE_Mod')
        return rdStr

    def Get_LTE_ResBlock(self):
        if self.ldir == 'UL':
            rdStr = self.query(f':CONF:LTE:{self.ldir}:SUBF0:ALL:RBC?')
        elif self.ldir == 'DL':
            rdStr = self.query(f':CONF:LTE:{self.ldir}:SUBF0:ALL0:RBC?')
        else:
            print('LDIR error in Get_LTE_RB')
        return rdStr

    def Get_LTE_ResBlockOffset(self):
        if self.ldir == 'UL':
            rdStr = self.query(f':CONF:LTE:{self.ldir}:SUBF0:ALL:RBOF?')
        elif self.ldir == 'DL':
            rdStr = self.query(f':CONF:LTE:{self.ldir}:SUBF0:ALL0:RBOF?')
        else:
            print('LDIR error in Get_LTE_RB')
        return rdStr

    #####################################################################
    ### Init LTE
    #####################################################################
    def Init_LTE(self):
        self.Set_Channel('LTE')

    def Init_LTE_Meas(self,sMeas):
        ### EMV; ESPectrum; ACLR; MCAClr; CACLr; MCESpectrum
        self.write('CONF:LTE:MEAS %s'%sMeas)

    def Init_LTE_SEM(self):
        self.Set_Channel('LTE')
        self.write(':CONF:LTE:MEAS ESP')

    #####################################################################
    ### VSA Settings
    #####################################################################
    def Set_LTE_AutoRef(self):
        #Assumes we are in LTE personality
        tick = datetime.now()
        self.Init_CCDF()
        self.Set_AttnAuto()
        self.Set_InitImm()
        Crest = self.queryFloat(f':CALC:STAT:CCDF:X1? P0_1;*WAI')
        ChPwr = self.queryFloat(f':CALC:STAT:RES? MEAN')
        refLvl = Crest + ChPwr
        self.Set_RefLevel(refLvl)
        self.Set_PreampToggle(ChPwr,-23)        #FSVA:-23  FSW:-27
        d = datetime.now() - tick
        print(f'Set_LTE_AutoRef: {d.seconds:3d}.{d.microseconds:06d}')

    def Set_LTE_CC(self,iCC):
        self.write(':CONF:LTE:NOCC %d'%iCC)
        self.cc = iCC

    def Set_LTE_ChBW(self,iBW):
        self.write(f':CONF:LTE:{self.ldir}:CC:BW BW{iBW:d}_00')

    def Set_LTE_Direction(self,sDir):
        # UL or DL
        if (sDir == "UL") or (sDir == "UP"):
            self.write(f':CONF:LDIR UL')
            self.ldir = 'UL'
        elif (sDir == "DL") or (sDir == "DOWN"):
            self.write(f':CONF:LDIR DL')
            self.ldir = 'DL'
        else:
            print("Set_5GNR_UL_Direction must be UL or DL")
        # print(self.ldir)

    def Set_LTE_Duplex(self,sDup):
        # TDD or FDD
        self.write(f':CONF:LTE:DUPL {sDup}')

    def Set_LTE_EVMUnit(self,sUnit):
        #DB or PCT
        self.write(':UNIT:EVM %s'%sUnit)

    def Set_LTE_SubFrameCount(self,dSubFrame):
        self.write(':SENS:LTE:FRAM:COUN:STAT ON')
        self.write(':SENS:LTE:FRAM:COUN:AUTO OFF')
        self.write(':SENS:LTE:FRAM:COUN %d'%dSubFrame)

    def Set_LTE_Modulation(self,iMod):
        self.write(f':CONF:LTE:{self.ldir}:SUBF0:ALL:MOD {iMod}')

    def Set_LTE_ResBlock(self,iRB):
        self.write(f':CONF:LTE:{self.ldir}:SUBF0:ALL:RBC {iRB}')

    def Set_LTE_ResBlockOffset(self,iRBO):
        self.write(f':CONF:LTE:{self.ldir}:SUBF0:ALL:RBOF %d'%iRBO)

    def Set_LTE_SweepTime(self,fSwpTime):
        if fSwpTime < 0.00201:
            fSwpTime = 0.0021
        self.write('SENS:SWE:TIME %f'%fSwpTime)  #Sweep/Capture Time

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    FSW = VSA()
    FSW.ldir = 'UL'
    FSW.jav_Open("192.168.1.109")
    FSW.Get_LTE_Direction()
    FSW.jav_IDN()



"""rssd.self.OTA.common.py driver test"""
import unittest
from rssd.OTA.Common import OTA

host = '192.168.1.40'              #Get local machine name

class TestGeneral(unittest.TestCase):
    def setUp(self):                      #run before each test
        self.ATSxxxx = OTA().open(host, type='test')

    def tearDown(self):                         #Run after each test
        self.assertEqual(self.ATSxxxx.SCPI_error(self)[0],'0')
        self.ATSxxxx.close()

    def test_OTA_Query(self):
        self.ATSxxxx.query('*IDN?')

if __name__ == '__main__':
#coverage run -a -m unittest -b -v test_HW_OTA_Common
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=4).run(suite)



"""RSSD Frequency Sweep VSA & VSG"""
from rssd.VSA           import VSA
from rssd.VSG           import VSG
from rssd.FileIO        import FileIO

freqOffset = 60e6
smwPwr      = -10
OFile = FileIO().makeFile(__file__)                 # Create Output File
SMW = VSG().jav_Open('192.168.58.114',OFile)        # Create Generator Object
FSW = VSA().jav_Open('192.168.58.109',OFile)        # Create Analyzer Object

OFile.write('SMWFreq,FreqOffset,SMWPwr,MkrBand,MkrCenter,MkrOffset,MkrDelta,SMWOff')
Freqs = range(int(7e9), int(9e9), int(100e6))     # Frequency List
SMW.Set_RFPwr(smwPwr)
SMW.Set_IQMod(0)
FSW.Set_Span(200e6)
FSW.Set_SweepTime(0.500)
FSW.Set_Trace_Detector('RMS')
FSW.Set_Mkr_Band(100e6)
for frq in Freqs:
    ## SMW Power ON
    SMW.Set_RFState(1)
    SMW.Set_Freq(frq)
    FSW.Set_SweepCont(0)
    FSW.Set_Freq(frq)
    FSW.Set_Autolevel()
    FSW.Set_InitImm()
    FSW.Set_Mkr_Freq(frq)
    MkrCenter = FSW.Get_Mkr_Y()
    MkrBandPr = FSW.Get_Mkr_Band()
    FSW.Set_Mkr_Freq(frq + freqOffset)
    MkrOffset = FSW.Get_Mkr_Y()

    ## SMW Power Off
    SMW.Set_RFState(0)
    FSW.Set_InitImm()
    MkrOff = FSW.Get_Mkr_Y()

    OFile.write(f'{frq},{freqOffset},{smwPwr},{MkrBandPr[1]},{MkrCenter},{MkrOffset},{MkrCenter-MkrOffset},{MkrOff}')

SMW.jav_Close()
FSW.jav_Close()



###############################################################################
### Rohde & Schwarz Automation for demonstration use.
### Title  : Timing SCPI Commands Example
### Author : mclim
### Date   : 2020.04.20
###
###############################################################################
### User Entry
###############################################################################
VSA_IP  = '192.168.1.109'
VSG_IP  = '192.168.1.114'
MeasTim = 100e-6
Freq    = 6e9
Avg     = 0

### Loops
Repeat  = 1
PwrArry = range(-50,0,2)

###############################################################################
### Code Overhead
###############################################################################
import timeit
from rssd.VSA.WLAN_K91      import VSA              #pylint: disable=E0611,E0401
from rssd.VSG.WLAN_K54      import VSG              #pylint: disable=E0611,E0401
from rssd.FileIO            import FileIO           #pylint: disable=E0611,E0401

OFile = FileIO().makeFile(__file__)
VSA = VSA().jav_Open(VSA_IP,OFile)                  #Create VSA Object
VSG = VSG().jav_Open(VSG_IP,OFile)                  #Create VSG Object

###############################################################################
### Instrument Init
###############################################################################
#VSA.Set_DisplayUpdate("OFF")
VSA.Set_Param_Couple_All()
VSA.Set_In_YIG('OFF')
VSA.Set_Freq(Freq)

VSA.Init_IQ()                                       #FSW IQ Channel
VSA.Set_IQ_Time(MeasTim)

VSA.Init_WLAN()
VSA.Set_WLAN_Standard('AC')
VSA.Set_WLAN_ChBW(0)
VSA.Set_WLAN_CaptureTime(2e-3)

VSG.Set_Freq(Freq)

###############################################################################
### Measure Time
###############################################################################
LoopParam   = 'Iter,Pwr'
TimeParam   = 'AlTime,MeasTime,TotalTIme'
AmpParam    = VSA.Get_Params_Amp(1)
SysParam    = VSA.Get_Params_System(1)
MeasData    = VSA.Get_Params_WLAN_EVM(1)
OFile.write(f'{LoopParam},{TimeParam},{AmpParam},{SysParam},{MeasData}')

tick0 = timeit.default_timer()
for i in range(Repeat):
    for SMWPwr in PwrArry:
        ### <thing we are timing>
        VSG.Set_RFPwr(SMWPwr)
        tick = timeit.default_timer()

        ### <AUTOLEVEL> ###
        VSA.Get_ChannelName()
        Fs = VSA.Get_IQ_SamplingRate()
        VSA.Init_IQ()                                           #FSW IQ Channel
        VSA.Set_IQ_SamplingRate(Fs)
        VSA.Set_IQ_Time(MeasTim)
        VSA.Set_Autolevel()
        VSA.Set_ChannelSelect(VSA.CurrCh)
        tockA = timeit.default_timer()
        ### </AUTOLEVEL> ###

        VSA.write(':INIT:CONT OFF')                             # Single Sweep
        VSA.query(':INIT:IMM;*OPC?')                            # Take Sweep
        EVM         = VSA.Get_Params_WLAN_EVM()
        tockB       = timeit.default_timer()
        ### <\thing we are timing>

        ALTime      = tockA - tick
        TotTime     = tockB - tick
        TestTime    = TotTime - ALTime

        LoopParam   = f'{i},{SMWPwr:5.2f}'
        TotTime     = f'{ALTime:2,.6f},{TestTime:2,.6f},{TotTime:2,.6f}'
        AmpParam    = VSA.Get_Params_Amp()
        SysParam    = VSA.Get_Params_System()
        MeasData    = EVM
        OutStr      = f'{LoopParam},{TotTime},{AmpParam},{SysParam},{MeasData}'
        OFile.write(OutStr)

SuiteTime = timeit.default_timer() - tick0
print(f'Total Test time : {SuiteTime:2,.6f}')
print(f'Time/Measurement: {SuiteTime/(Repeat*len(PwrArry)):2,.6f}')

###############################################################################
### Cleanup Automation
###############################################################################
OFile.write("\n")
VSA.jav_Close()



###############################################################################
### Rohde & Schwarz Automation for demonstration use.
### Purpose: Vector Signal Explorer K96 Functions
### Author : Martin C Lim
### Date   : 2018.04.27
###############################################################################
import time            #EVM Wait
from rssd.VSE.Common import VSE

class VSE(VSE):
    """ Rohde & Schwarz VSE K96 Object """
    def __init__(self):
        super(VSE, self).__init__()
        self.Model = 'VSE'

    ###########################################################################
    ### VSE General Settings
    ##########################################################################
    def Init_K96(self):
        self.Set_Channel('OFDMVSA')
        # time.sleep(10)
        self.write("SENS:DEM:FSYN DATA")                # Pilot and Data Aided
        self.write("SENS:TRAC:TIME ON")                 # Timing tracking ON
        self.write("SENS:TRAC:LEV ON")                  # Leveltracking ON

    def Set_K96_BurstSearch(self,sState):
        self.write('DEM:FORM:BURS %s;*WAI'%sState)      #ON|OFF|1|0

    def Set_K96_File_Config(self,sFile):
        self.write("MMEM:LOAD:CFGF '%s';*WAI"%sFile)

    def Set_K96_FilterAdjustable(self,sState):
        self.write('INP:FILT:CHAN:STAT %s'%sState)      #ON|OFF|1|0

    def Set_K96_Frames(self,iNum):
        self.write('SENS:DEM:FORM:MAXF %d'%iNum)        #Number of frames to analyze

    def Set_K96_FSWIPAdd(self,sIP):                     #For FS-K96
        self.write('CONF:ADDR "TCPIP::%s"'%sIP)         #FSW IP Address

    def Set_K96_OFDMSymbols(self,iNum):
        self.write('SENS:DEM:FORM:NOFS %d'%iNum)        #FSW IP Address

    ###########################################################################
    ### Helper Functions
    ###########################################################################
    def K96_EVM_Wait(self):
        EVM = ''                                        #init EVM read value
        t0 = time.time()
        self.write('INIT:IMM')                          #Start Measurement
        while (EVM == ''):                              #Loop until EVM present
            delta = (time.time() - t0)
            try:
                EVM = self.Get_EVM()
            except:
                pass
            if delta > 10:                              #EVM_Wait Timeout
                break
        print("K96_EVM_Wait: %.3f sec"%(delta))
        self.Get_EVM()                                  #Flush buffer of NaN

    def K96_EVM_AutoCal(self):                          #pylint: disable=R0915,R0912
        #######################################################################
        ### Code Settings
        #######################################################################
        debug=0
        Backoff = 0
        EVM_Wind = -0.1                                 #EVM can degrade by this amount.
                                                        #EVM Repeatability indicator
        #######################################################################
        ### Code Start
        #######################################################################
        if debug==1: print('    Autolvl EVM: ')
        self.Set_Autolevel()
        EVM_Curr = self.Get_EVM()                       #Set initial RefLvl & Attn
        RefLvl = self.Get_RefLevel()
        MAttn  = self.Get_AttnMech()
        if debug==1: print("        Ref:%.2f MAttn:%.0f EVM:%.2f"%(RefLvl, MAttn, EVM_Curr))

        #######################################################################
        ### Attn Sweep
        #######################################################################
        if debug==1: print('    Attenu Swp: ')
        EVM_Prev = 1.00
        i=0
        self.Set_Autolevel()                            #Manually Set RefLvl & Attn
        while (i <= MAttn) & (i < 30):                  #pragma: no cover
            MechAttn = MAttn - i
            self.Set_AttnMech(MAttn - i)
            EVM_Curr = self.Get_EVM()
            if debug==1: print("        Ref:%.2f MAttn:%.0f EVM:%.2f"%(RefLvl, MechAttn, EVM_Curr))
            if EVM_Curr =='NAN':
                print("EVM NAN")
                break

            Diff = EVM_Prev - EVM_Curr                 #Positive = improvedEVM
            if (Diff > EVM_Wind):
                EVM_Prev = EVM_Curr
                i = i + 1
            else:
                if debug==1: print("        Break")
                i = i - 1                                       #Previous value
                break
        MechAttn = MAttn - i + Backoff                          #MechAttn Used for next step
        if (MechAttn < 0):
            MechAttn= 0
        self.Set_AttnMech(MechAttn)

        #######################################################################
        ### Ref Sweep
        #######################################################################
        #print('    RefLvl Swp: ')
        EVM_Prev = 1.00
        i=0
        self.Set_Autolevel()                                    #Manually Set RefLvl & Attn
        for x in range(0):                                      #pragma: no cover
            self.Set_RefLevel(RefLvl - i)
            EVM_Curr = self.Get_EVM()
            if debug==1: print("        Ref:%.2f MAttn:%.0f EVM:%.2f"%(RefLvl-i, MechAttn, EVM_Curr))
            if EVM_Curr =='NAN':
                print("EVM NAN")
                break

            Diff = EVM_Prev - EVM_Curr                          #Positive = improvedEVM
            if (Diff > EVM_Wind):
                EVM_Prev = EVM_Curr
                i = i + 1
            else:
                i = i - 1                                       #Previous value
                self.Set_RefLevel(RefLvl - i)
                break
        self.K96_EVM_Wait()

###############################################################################
### Run if Main
###############################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    VSE = VSE()
    VSE.jav_Open("127.0.0.1")
    VSE.Set_DisplayUpdate('ON')
    VSE.Init_K96()
    #VSE.Get_EVM_Params()
    #VSE.Set_InitImm()
    VSE.jav_ClrErr()



"""rssd.VSA.common test
"""
host = '192.168.1.108'                              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSA.Common        import VSA              #pylint: disable=E0611,E0401

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #run before each test
        self.FSW = VSA().jav_OpenTest(host)
        self.FSW.Init_Spectral()

    def tearDown(self):                             #Run after each test
        self.assertEqual(self.FSW.jav_Error()[0],'0')
        self.FSW.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_FSW_Autolevel(self):
        self.FSW.Set_Autolevel()
        self.FSW.Set_Autolevel_IFOvld()

    def test_FSW_ACLR(self):
        self.FSW.Init_ACLR()
        self.FSW.Set_ACLR_AdjBW(95e6)
        self.FSW.Set_ACLR_AdjSpace(100e6)
        self.FSW.Set_ACLR_CHBW(100e6)
        self.FSW.Set_ACLR_NumAdj(2)
        self.FSW.Get_ACLR()

    def test_FSW_ChannelManagement(self):
        if self.FSW.connected: getVal = self.FSW.Get_ChannelName()
        getVal = self.FSW.Get_Channels()
        self.FSW.Init_IQ()
        self.FSW.Set_ChannelName('IQ','IQ_Test')
        self.FSW.Set_ChannelSelect('Spectrum')

    def test_FSW_Connect(self):
        self.FSW.jav_IDN()
        if self.FSW.connected: self.assertEqual(self.FSW.Make,"Rohde&Schwarz")  # Valuecompare

    def test_FSW_Ex_SpectralSettings(self):
        self.FSW.Set_Freq(1e6)
        self.FSW.Set_RefLevel(10)
        self.FSW.Set_ResBW(1e6)
        self.FSW.Set_VidBW(1e6)
        self.FSW.Set_Span(100e6)
        self.FSW.Set_AttnMech(10)
        self.FSW.Get_IFOvld()
        self.FSW.Get_ACLR()
        self.FSW.Set_DisplayUpdate('ON')
        self.FSW.Set_Ref_Source('INT')

    def test_FSW_Ex_SpectralSettings_Auto(self):
        self.FSW.Set_Freq(1e6)
        self.FSW.Set_ResBW(0)
        self.FSW.Set_VidBW(0)
        self.FSW.Set_Span(100e6)
        self.FSW.Set_Autolevel()
        self.FSW.Set_AttnAuto()
        self.FSW.Get_IFOvld()
        self.FSW.Get_ACLR()
        self.FSW.Set_DisplayUpdate('ON')

    def test_FSW_Equalizer(self):
        self.FSW.Set_EQ_File('Test')
        self.FSW.Set_EQ_State('ON')
        self.FSW.Set_EQ_State('OFF')

    def test_FSW_Freq(self):
        self.FSW.Set_FreqStart(10e6)
        self.FSW.Set_FreqStop(100e6)
        # self.FSW.Set_FreqStep(1e5)
        self.FSW.Set_Freq(100e6)
        rdStr = self.FSW.Get_Freq()
        #var = input("Please enter something: ")
        if self.FSW.connected: self.assertEqual(100e6,rdStr)                    # Valuecompare

    def test_FSW_GetParams(self):
        nullVal = self.FSW.Get_Params_Amp(1)
        nullVal = self.FSW.Get_Params_Amp()
        nullVal = self.FSW.Get_Params_Sweep(1)
        nullVal = self.FSW.Get_Params_Sweep()
        nullVal = self.FSW.Get_Params_Trace(1)
        nullVal = self.FSW.Get_Params_Trace()
        nullVal = self.FSW.Get_Params_System(1)
        nullVal = self.FSW.Get_Params_System()
        nullVal = self.FSW.Get_Params_MkrBand(1)
        nullVal = self.FSW.Get_Params_MkrBand()
        nullVal = self.FSW.Get_Params(1,1,1,1,0)

    def test_FSW_GetScreenshot(self):
        getVal = self.FSW.Get_Screenshot()

    def test_FSW_GetTrace(self):
        getVal = self.FSW.Get_Trace_Data()

    def test_FSW_Input(self):
        self.FSW.Set_Freq(1e9)
        self.FSW.Set_Input('RF')
        self.FSW.Set_In_YIG('ON')
        self.FSW.Set_In_YIG('OFF')
        self.FSW.Set_In_HPFilter('ON')

    def test_FSW_Marker_dB(self):
        self.FSW.Set_Mkr_AllOff()
        self.FSW.Set_Mkr_Peak()
        self.FSW.Set_Mkr_On(2)
        self.FSW.Set_Mkr_Next(2)
        getVal = self.FSW.Get_Mkr_Freq()
        getVal = self.FSW.Get_Mkr_XY()
        getVal = self.FSW.Get_Mkr_Y()

    def test_FSW_Marker_BanddB(self):
        self.FSW.Set_Mkr_AllOff()
        self.FSW.Set_Mkr_BandSetRef()
        self.FSW.Set_Mkr_BandDelta(1e9)
        self.FSW.Get_Mkr_Band()

    def test_FSW_Marker_Noise(self):
        self.FSW.Get_Mkr_Noise()

    def test_FSW_Marker_Time(self):
        self.FSW.Set_Span(0)
        self.FSW.Set_SweepTime(0.010)
        self.FSW.Set_Mkr_Time(0.001)
        rdStr = self.FSW.Get_Mkr_TimeDomain()
        if self.FSW.connected: self.assertEqual(rdStr, 0.001)
        self.FSW.Set_Span(900e6)
        self.FSW.Set_SweepTime(0)                           #Auto Sweeptime

    def test_FSW_Trigger(self):
        self.FSW.Set_Trig1_Source('IMM')
        self.FSW.Set_Trig2_Dir('OUT')
        self.FSW.Set_Trig2_OutType('DEV')
    
    def test_FSW_TraceAvg(self):
        self.FSW.Set_Trace_Avg('LIN')
        self.FSW.Set_Trace_AvgCount(10)
        self.FSW.Set_Trace_Detector('RMS')
        self.FSW.Set_Trace_Mode('WRIT')
        self.FSW.Set_Trace_AvgOff()

    def test_FSW_Sweep(self):
        self.FSW.Set_SweepPoints(1001)
        self.FSW.Set_SweepType('AUTO')
        self.FSW.Set_SweepOpt('AUTO')

    def test_FSW_Sys_Overload(self):
        getVal = self.FSW.Get_Ovld_Stat()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    # coverage run -a -m unittest -b -v test_HW_VSA_Common
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""rssd.VSA.ADemod_K7 test
"""
host = '10.0.0.13'                              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VNA.Common        import VNA              #pylint: disable=E0611,E0401

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #run before each test
        self.VNA = VNA().jav_OpenTest(host)

    def tearDown(self):                             #Run after each test
        self.assertEqual(self.VNA.jav_Error()[0],'0')
        self.VNA.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_VNA_Convert(self):
        self.VNA.__complex2dB__([1,1])
        self.VNA.__complex2phase__([1,1])
        self.VNA.__complex2dBm__([1,1])

    def test_VNA_Markers(self):
        self.VNA.Set_Mkr_Coupled(1)
        self.VNA.Set_Mkr_Coupled(0)
        self.VNA.Set_Mkr_Frq(2.4e9,1)
        self.VNA.Set_Mkr_Frq(2.5e9,2)

    def test_VNA_PowerCal_Tx(self):
        self.VNA.Set_FreqStart(1e9)
        self.VNA.Set_FreqStop(6e9)
        self.VNA.Set_SweepPoints(601)
        self.VNA.Set_Pwrcal_Init()
        self.VNA.Set_Pwrcal_Tolerance(0.1)
        self.VNA.Set_Pwrcal_NumReading(10)
        self.VNA.Set_Pwrcal_Rx(1,2)
        self.VNA.Set_Pwrcal_Measure(2)                      #Initiate Power cal
        getVal = self.VNA.Get_Pwrcal_State()                #Pwr Cal Tx State
        getVal = self.VNA.Get_Pwrcal_Rx_State()             #Pwr Cal Rx State

    def test_VNA_PowerSweep(self):
        self.VNA.Set_PowerStart(-60)
        self.VNA.Set_PowerStop(-10)

    def test_VNA_SParam(self):
        self.VNA.Set_SweepCont(0)
        self.VNA.Set_Trace_MeasAdd_SParam(1,1)              #S11 Measurement
        self.VNA.Set_Trace_MeasAdd_SParam(2,1)              #S21 Measurement
        self.VNA.Set_FreqStart(100e6)
        self.VNA.Set_FreqStop(200e6)
        self.VNA.Set_SweepPoints(1001)
        self.VNA.Set_IFBW(1000)
        self.VNA.Set_InitImm()
        self.VNA.Get_Trace_Names()

    def test_VNA_SaveData(self):
        self.VNA.Save_Cal("TestCalFile")
        self.VNA.Save_Screen('TestPicture')
        self.VNA.Save_State('TestState')
        self.VNA.Save_Trace_CSV('TestTraceCSV')
        self.VNA.Save_Trace_SxP('TestTraceSxP')

    def test_VNA_Sweep(self):
        self.VNA.Set_SweepCont(1)
        self.VNA.Set_SweepTime(1000)
        self.VNA.Set_SweepTime(0)
        self.VNA.Set_Trace_AvgCount(10)
        self.VNA.Set_Trace_Avg('ON')

    def test_VNA_TraceChanges(self):
        self.VNA.Set_Trace_MeasAdd_BWave(1,2)
        self.VNA.Set_Trace_MeasAdd_PwrMtr(1)
        self.VNA.Set_Trace_MeasAdd_SParam(2,1)
        self.VNA.Set_Trace_Select('S21')
        self.VNA.Set_Trace_MeasDel('S21')
        self.VNA.Set_Trace_DelAll()
        self.VNA.Set_Trace_MeasAdd_AWave(1,2)

    def test_VNA_TraceGet(self):
        self.VNA.Get_Trace_Data()
        self.VNA.Get_Trace_Data_All()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
#coverage run -a -m unittest -b -v test_HW_VNA_Common
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



###############################################################################
### Purpose: rssd.VSA.LTE_K100 test
###              _   ___        __  _____         _
###             | | | \ \      / / |_   _|__  ___| |_
###             | |_| |\ \ /\ / /    | |/ _ \/ __| __|
###             |  _  | \ V  V /     | |  __/\__ \ |_
###             |_| |_|  \_/\_/      |_|\___||___/\__|
###             Please connect instrument prior 2 test
###############################################################################
### User Entry
###############################################################################
host = '192.168.1.109'                              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSA.LTE_K100      import VSA

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #run before each test
        self.FSW = VSA().jav_OpenTest(host)
        self.FSW.Init_LTE()

    def tearDown(self):                             #Run after each test
        self.assertEqual(self.FSW.jav_Error()[0],'0')
        self.FSW.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_FSW_LTE_AutoRef(self):
        self.FSW.Set_LTE_AutoRef()

    def test_FSW_LTE_Get_DL(self):
        self.FSW.Set_LTE_Direction('DL')
        self.FSW.Set_LTE_Duplex('FDD')
        nullVal = self.FSW.Get_LTE_Direction()
        nullVal = self.FSW.Get_LTE_Duplex()
        nullVal = self.FSW.Get_LTE_ChBW()
        nullVal = self.FSW.Get_LTE_ResBlock()           # Need to test DL
        nullVal = self.FSW.Get_LTE_ResBlockOffset()     # Need to test DL
        nullVal = self.FSW.Get_LTE_Modulation()         # Need to test DL

    def test_FSW_LTE_Get_UL(self):
        self.FSW.Set_LTE_Direction('UL')
        self.FSW.Set_LTE_Duplex('FDD')
        nullVal = self.FSW.Get_LTE_Direction()
        nullVal = self.FSW.Get_LTE_Duplex()
        nullVal = self.FSW.Get_LTE_ChBW()
        nullVal = self.FSW.Get_LTE_ResBlock()           # Need to test DL
        nullVal = self.FSW.Get_LTE_ResBlockOffset()     # Need to test DL
        nullVal = self.FSW.Get_LTE_Modulation()         # Need to test DL

    def test_FSW_LTE_CC(self):
        self.FSW.Set_LTE_CC(2)
        getVal = self.FSW.Get_LTE_CC()
        if self.FSW.connected: self.assertEqual(getVal,'2')
        self.FSW.Set_LTE_CC(1)
        getVal = self.FSW.Get_LTE_CC()
        if self.FSW.connected: self.assertEqual(getVal,'1')

    def test_FSW_LTE_Direction(self):
        self.FSW.Set_LTE_Direction('UL')
        getVal = self.FSW.Get_LTE_Direction()
        if self.FSW.connected: self.assertEqual(getVal,'UL')
        self.FSW.Set_LTE_Direction('DL')
        getVal = self.FSW.Get_LTE_Direction()
        if self.FSW.connected: self.assertEqual(getVal,'DL')

    def test_FSW_LET_Meas(self):
        self.FSW.Get_LTE_EVMParams()
        self.FSW.Get_EVM_n_Params()
        self.FSW.Get_EVM()
        self.FSW.Get_LTE_CrestFactor()
        self.FSW.Get_LTE_ChPwr()

    def test_FSW_LTE_Set_UL(self):
        self.FSW.Set_Freq(2e9)
        self.FSW.Set_LTE_EVMUnit('DB')
        self.FSW.Set_LTE_Duplex('TDD')
        self.FSW.Set_LTE_Duplex('FDD')
        self.FSW.Set_LTE_SubFrameCount(2)
        self.FSW.Set_LTE_Direction('UL')
        self.FSW.Set_LTE_ChBW(20)
        self.FSW.Set_LTE_ResBlock(66)
        self.FSW.Set_LTE_ResBlockOffset(0)
        self.FSW.Set_LTE_Modulation('QPSK')

    def test_FSW_LTE_SEM(self):
        self.FSW.Init_LTE_SEM()
        # self.FSW.Get_LTE_SEM()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
#coverage run -a -m unittest -b -v test_HW_VSA_LTE
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""Radio Communication Tester(RCT) General Purpose RF(GPRF) Functions"""
import struct                                   # For IQ Manipulation
from rssd.RCT.Common import RCT                 #pylint: disable=E0611,E0401

class RCT(RCT):                                 #pylint: disable=E0102
    """ Rohde & Schwarz Radio Comm Tester Object """
    def __init__(self):
        super(RCT, self).__init__()
        self.Model = "CMW-GPRF"

    ###########################################################################
    ### RCT Get Functions
    ###########################################################################
    def Get_IQR_srate(self):
        rdStr = self.queryFloat('FETCh:GPRF:MEAS:IQRecorder:SRATe?')
        return rdStr

    def Get_IQR_data_ASCII(self):
        self.write('FORMAT:BASE:DATA ASCII')
        rdStr = self.query('FETC:GPRF:MEAS:IQR?')
        return rdStr

    def Get_IQR_data_Bin(self):
        self.write('FORMAT:BASE:DATA REAL,32')
        self.write('FETC:GPRF:MEAS:IQR?')
        rdStr = self.K2.read_raw()
        numBytes = int(chr(rdStr[3]))           # Number of Bytes
        numIQ    = int(rdStr[2:2+numBytes])
        IQBytes  = rdStr[(numBytes+4):-1]       # Remove Header
        IQAscii  = struct.unpack("<" + 'f' * int(numIQ/4),IQBytes)          #pylint: disable=W0612
        self.write('FORMAT:BASE:DATA ASCII')
        return IQBytes

    ###########################################################################
    ### RCT Init Functions
    ###########################################################################
    def Init_Meas_IQCapture(self,port=1):
        self.write('INIT:GPRF:MEAS:IQR')
        self.Set_Meas_Port(port)
        self.write(f'CONFigure:GPRF:MEAS:IQRecorder:FORMat IQ')

    ###########################################################################
    ### RCT Set Functions
    ###########################################################################
    def Set_Meas_Freq(self,fFreq):                                          #Val
        self.write(f'CONF:GPRF:MEAS:RFS:FREQ {fFreq}')

    def Set_IQR_Bandwidth(self,iBW):
        """1000:1250 | 500:625 | 250:312.5 160/40/10:Unsupported"""
        self.write(f'CONFigure:GPRF:MEAS:IQRecorder:FILTer:BANDpass:BWIDth {iBW} MHz')

    def Set_IQR_filename(self,sFile):
        """IQ Output Filename"""
        self.write(f'CONFigure:GPRF:MEAS:IQRecorder:IQFile "@USERDATA/iqrecorder/{sFile}.iqw"')
        self.Set_IQR_filename_state('ON')

    def Set_IQR_filename_state(self,sState):
        """OFF | ON | ONLY"""
        self.write(f'CONFigure:GPRF:MEAS:IQRecorder:WTFile {sState}')

    def Set_IQR_InitImm(self):
        self.query('STOP:GPRF:MEAS:IQR;*OPC?')
        self.query('INIT:GPRF:MEAS:IQR;*OPC?')
        self.Set_IQR_filename_state('OFF')
        # self.query('ABOR:GPRF:MEAS:IQR;*OPC?')

    def Set_IQR_Length(self,iLength):
        self.write(f'CONFigure:GPRF:MEAS:IQRecorder:CAPTure 1, {iLength}')
        # self.write(f'CONFigure:GPRF:MEAS:IQRecorder:FILTer:TYPe GAUSs')

    def Set_IQR_SamplingRate(self,fFreq):
        """Sampling Rate, MHz"""
        # 10/40/160 BW ->  Unsupported
        fFreq = float(fFreq)
        if fFreq < 156.25:
            print('Set_IQR_SamplingRate Frequency below 156.25 not supported')
        elif fFreq < 312.5:             #  250 MHz BW -->  312.5 MHz
            self.Set_IQR_Bandwidth(250)
            ratio = fFreq/312.5
            self.Set_IQR_SamplingRate_Ratio(ratio)
        elif fFreq < 625:               #  500 MHz BW -->  625 MHz
            self.Set_IQR_Bandwidth(500)
            ratio = fFreq/625
            self.Set_IQR_SamplingRate_Ratio(ratio)
        elif fFreq < 1250:              # 1000 MHz BW --> 1250 MHz
            self.Set_IQR_Bandwidth(1000)
            ratio = fFreq/1250
            self.Set_IQR_SamplingRate_Ratio(ratio)
        else:
            print('Frequency too high')

    def Set_IQR_SamplingRate_Ratio(self,fRatio):
        """Fs = BP Filt BW * Ratio"""
        self.write(f'CONFigure:GPRF:MEAS:IQRecorder:RATio {fRatio}')

    def Set_IQR_timeout(self,fTime):
        """Timeout, sec"""
        self.write(f'CONFigure:GPRF:MEAS:IQRecorder:TOUT {fTime}')

    def Set_IQR_Time(self,fTime):
        """Capture Time, sec"""
        Fs     = self.Get_IQR_srate()
        numIQ  = int(Fs * fTime)
        self.Set_IQR_Length(numIQ)

    def Set_IQR_Trig_Slope(self,sSlope):
        """Trigger Slope: REDG | FEDGe"""
        self.write(f'TRIGger:GPRF:MEAS:IQRecorder:SLOPe {sSlope}')

    def Set_IQR_Trig_Source(self,sSource):
        """Trigger Source 'IF Power'; 'Free Run' """
        self.write(f'TRIGger:GPRF:MEAS<i>:IQRecorder:SOURce {sSource}')

    def Set_IQR_Trig_Threshold(self,sdBm):
        """Trigger Threshold, dBm"""
        self.write(f'TRIGger:GPRF:MEAS:IQRecorder:THReshold {sdBm}')

    def Set_IQR_Trig_Timeout(self,iTime):
        """Trigger Timeout, sec"""
        self.write(f'TRIGger:GPRF:MEAS:IQRecorder:TOUT {iTime}')

    def Set_IQR_Trig_MinGap(self,iTime):
        """Trigger Timeout, sec"""
        self.write(f'TRIGger:GPRF:MEAS:IQRecorder:MGAP {iTime}')

    def Set_IQR_Units(self,sUnit):
        """VOLT (Float32) | RAW"""
        self.write(f'CONFigure:GPRF:MEAS:IQRecorder:MUNit {sUnit}')

###############################################################################
### Run if Main
###############################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    CMP = RCT()
    CMP.jav_Open("192.168.1.160")
    # CMP.Set_IQR_timeout(1)
    # CMP.Init_Meas_IQCapture()
    # CMP.Set_IQR_Time(0.0125)
    # CMP.Set_IQR_SamplingRate(245.76)
    print(CMP.Get_IQR_data_ASCII()[0:50])
    rd = (CMP.Get_IQR_data_Bin())
    print(rd)
    CMP.jav_ClrErr()
    CMP.jav_Close()



""" Vector Signal Explorer Common Functions"""
from rssd.VSA.Common import VSA

class VSE(VSA):
    """ Rohde & Schwarz Vector Signal Explorer Object """
    def __init__(self):
        super(VSE,self).__init__()
        self.Model = "VSE"

    def Set_File_InputIQT(self,sFilename):
        ABW = '10MHz'
        IQCh = '2'
        self.write("INST:BLOC:CHAN:FILE:IQT '%s',%s,%s"%(sFilename,ABW,IQCh))

    def Set_File_InputIQW(self,Fs,sFileName='\\file.iqw'):
        abw = 0.8*float(Fs)
        val = "'%s',%d,%d,IQIQ"%(sFileName,abw,Fs)
        print("VSE_Common  :" + val)
        self.write("INST:BLOC:CHAN:FILE:IQW " + val)

    def Set_Group(self,sGroup):
        GroupList = self.query('INST:BLOC:LIST?').split(',')
        print("Grup:%s"%GroupList)
        if ("'" + sGroup + "'") in GroupList:
            pass
        else:
            self.write(":INST:BLOC:CRE '%s'"%(sGroup))
        self.write("INST:BLOC:USE 1, '%s'"%sGroup)

    def Set_Input(self,sType):
        """RF|FILE"""
        self.write(f'INP:SEL {sType}')

    def Set_SweepCont(self, sState):
        """0 | 1 """
        if sState in (1, '1', 'ON'):
            self.write('INIT:SEQ:MODE CONT')            # Continuous Sweep
        elif sState in (0, '0', 'OFF'):
            self.write('INIT:SEQ:MODE SING')            # Single Sweep
        else:
            print('Set_SweepCont error')


#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":                              # This won't be run when imported
    VSE = VSE()
    VSE.jav_Open("127.0.0.1")                           # Prints IDN String
    VSE.Set_SweepCont(0)
    VSE.jav_ClrErr()
